@@ -12,7 +12,7 @@ use feature ();
 package Zydeco;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.510';
+our $VERSION   = '0.511';
 
 use Keyword::Simple ();
 use PPR;
@@ -21,6 +21,13 @@ use Exporter::Shiny our @EXPORT = qw( version authority overload );
 use Devel::StrictMode qw(STRICT);
 use Types::Standard qw( is_HashRef is_Str );
 
+my $decomment = sub {
+	require Carp;
+	Carp::carp("Cannot remove comments within a type constraint; please upgrade perl");
+	return $_[0];
+};
+$decomment = \&PPR::decomment if $] >= 5.016;
+
 BEGIN {
 	package Zydeco::_Gather;
 	my %gather;
@@ -223,14 +230,14 @@ our $GRAMMAR = qr{
 		
 			(?&MxpSimpleTypeSpec)
 			(?:
-				\s*\&\s*
+				(?&PerlOWS) \& (?&PerlOWS)
 				(?&MxpSimpleTypeSpec)
 			)*
 			(?:
-				\s*\|\s*
+				(?&PerlOWS) \| (?&PerlOWS)
 				(?&MxpSimpleTypeSpec)
 				(?:
-					\s*\&\s*
+					(?&PerlOWS) \& (?&PerlOWS)
 					(?&MxpSimpleTypeSpec)
 				)*
 			)*
@@ -250,7 +257,7 @@ our $GRAMMAR = qr{
 				(?&PerlVariable) | (\*(?&PerlIdentifier))
 			)
 			(?:                                           # CAPTURE:postamble
-				\? | ((?&PerlOWS)=(?&PerlOWS)(?&PerlTerm))
+				\? | ((?&PerlOWS)=(?&PerlOWS)(?&PerlScalarExpression))
 			)?
 		)#</MxpSignatureElement>
 		
@@ -320,6 +327,12 @@ our $GRAMMAR = qr{
 			)*
 		)#</MxpCompactRoleList>
 		
+		(?<MxpBlockLike>
+		
+			(?: (?&PerlBlock) ) |
+			(?: [=] (?&PerlOWS) (?&PerlScalarExpression) (?&PerlOWS) [;] )
+		)#</MxpBlockLike>
+		
 		(?<MxpIncludeSyntax>
 		
 			(?&PerlOWS)
@@ -545,7 +558,7 @@ our $GRAMMAR = qr{
 				[)]
 			)?
 			(?&PerlOWS)
-			(?: (?&PerlBlock) )                           # CAPTURE:code
+			(?: (?&MxpBlockLike) )                        # CAPTURE:code
 			(?&PerlOWS)
 		)#</MxpMethodSyntax>
 		
@@ -568,7 +581,7 @@ our $GRAMMAR = qr{
 				[)]
 			)?
 			(?&PerlOWS)
-			(?: (?&PerlBlock) )                           # CAPTURE:code
+			(?: (?&MxpBlockLike) )                        # CAPTURE:code
 			(?&PerlOWS)
 		)#</MxpMultiSyntax>
 		
@@ -589,7 +602,7 @@ our $GRAMMAR = qr{
 				[)]
 			)?
 			(?&PerlOWS)
-			(?: (?&PerlBlock) )                           # CAPTURE:code
+			(?: (?&MxpBlockLike) )                        # CAPTURE:code
 			(?&PerlOWS)
 		)#</MxpModifierSyntax>
 		
@@ -612,7 +625,7 @@ our $GRAMMAR = qr{
 				[)]
 			)?
 			(?&PerlOWS)
-			(?: (?&PerlBlock) )                           # CAPTURE:code
+			(?: (?&MxpBlockLike) )                        # CAPTURE:code
 			(?&PerlOWS)
 		)#</MxpFactorySyntax>
 		
@@ -645,7 +658,7 @@ our $GRAMMAR = qr{
 				(?&PerlBlock)|(?&PerlIdentifier)|(?&PerlString)
 			)
 			(?&PerlOWS)
-			(?: (?&PerlBlock) )?                          # CAPTURE:code
+			(?: (?&MxpBlockLike) )?                       # CAPTURE:code
 			(?&PerlOWS)
 		)#</MxpCoerceSyntax>
 		
@@ -735,7 +748,7 @@ sub _handle_signature_list {
 		}
 		elsif ($sig =~ /^((?&MxpTypeSpec)) $GRAMMAR/xso) {
 			my $type = $1;
-			$parsed[-1]{type}          = $type;
+			$parsed[-1]{type}          = ($type =~ /#/) ? $type->$decomment : $type;
 			$parsed[-1]{type_is_block} = 0;
 			$sig =~ s/^\Q$type//xs;
 			$sig =~ s/^((?&PerlOWS)) $GRAMMAR//xso;
@@ -768,7 +781,7 @@ sub _handle_signature_list {
 			$parsed[-1]{optional} = 1;
 			$sig =~ s/^\?((?&PerlOWS)) $GRAMMAR//xso;
 		}
-		elsif ($sig =~ /^=((?&PerlOWS))((?&PerlTerm)) $GRAMMAR/xso) {
+		elsif ($sig =~ /^=((?&PerlOWS))((?&PerlScalarExpression)) $GRAMMAR/xso) {
 			my ($ws, $default) = ($1, $2);
 			$parsed[-1]{default} = $default;
 			$sig =~ s/^=\Q$ws$default//xs;
@@ -951,6 +964,11 @@ sub _handle_factory_keyword {
 		$optim = 1 if $attr =~ /^:optimize\b/;
 	}
 	
+	if (defined $code and $code =~ /^=(.+)$/s) {
+		$code  = "{ $1 }";
+		$optim = 1;
+	}
+	
 	if ($via) {
 		return sprintf(
 			'q[%s]->_factory(%s, \\(%s));',
@@ -985,12 +1003,17 @@ sub _handle_factory_keyword {
 sub _handle_method_keyword {
 	my $me = shift;
 	my ($name, $code, $has_sig, $sig, $attrs) = @_;
-	
+
 	my $optim;
 	for my $attr (@$attrs) {
 		$optim = 1 if $attr =~ /^:optimize\b/;
 	}
 	
+	if (defined $code and $code =~ /^=(.+)$/s) {
+		$code  = "{ $1 }";
+		$optim = 1;
+	}
+	
 	my $lex_name;
 	if (defined $name and $name =~ /^\$(.+)$/) {
 		$lex_name = $name;
@@ -1067,6 +1090,11 @@ sub _handle_multimethod_keyword {
 		}
 	}
 	
+	if (defined $code and $code =~ /^=(.+)$/s) {
+		$code  = "{ $1 }";
+		$optim = 1;
+	}
+	
 	if ($has_sig) {
 		my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $me->_handle_signature_list($sig);
 		my $munged_code = sprintf('sub { my($self,%s)=(shift,@_); %s; my $class = ref($self)||$self; do %s }', $signature_var_list, $extra, $code);
@@ -1100,6 +1128,14 @@ sub _handle_modifier_keyword {
 		$optim = 1 if $attr =~ /^:optimize\b/;
 	}
 	
+	if (defined $code and $code =~ /^=(.+)$/s) {
+		$code  = "{ $1 }";
+		$optim = 1;
+	}
+	
+	# MooX::Press cannot handle optimizing method modifiers
+	$optim = 0;
+	
 	my @names = $me->_handle_name_list($names);
 	
 	my $processed_names =
@@ -2170,7 +2206,7 @@ sub _include {
 #{
 #	package Zydeco::Anonymous::Package;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.510';
+#	our $VERSION   = '0.511';
 #	use overload q[""] => sub { ${$_[0]} }, fallback => 1;
 #	sub DESTROY {}
 #	sub AUTOLOAD {
@@ -2181,7 +2217,7 @@ sub _include {
 #	
 #	package Zydeco::Anonymous::Class;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.510';
+#	our $VERSION   = '0.511';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	sub new {
 #		my $me = shift;
@@ -2194,12 +2230,12 @@ sub _include {
 #	
 #	package Zydeco::Anonymous::Role;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.510';
+#	our $VERSION   = '0.511';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	
 #	package Zydeco::Anonymous::ParameterizableClass;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.510';
+#	our $VERSION   = '0.511';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	sub generate_package {
 #		my $me  = shift;
@@ -2213,7 +2249,7 @@ sub _include {
 #
 #	package Zydeco::Anonymous::ParameterizableRole;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.510';
+#	our $VERSION   = '0.511';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	sub generate_package {
 #		my $me  = shift;
